%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

function [p,T,Xgc] = psAdiabaticMix(p,V1,T1,V2,T2,Xgc1,Xgc2,stopFrac,Tinit)
  
% Initializes gas exchange phase. This is an adiabatic mixing
% process. (Se Chapt4 Energy and mass balance for open systems in Lars
% Erikssons text.)
% The theory is that Gibbs-Daltons law holds etc.  Things that are
% assumed 
% * Energy conservation
% * Mass conservation
% * Idal gas law
% * Gibbs-Daltons law
% * End composition is homogene (same temperature)
% This gives:
% p is not supposed to change but that might however be the case
% m1*u1(T1) + m2*u2(T2) = m3*u3(T3)
% h = u + pv = u+RT => u = h-RT
% Solve therefore the following eq with Newton-Rhappson
% U = m1*(h1(T1)-R1(T1)*T1)+m2*(h2(T2)-R2(T2)*T2)  
% U = m3*(h3(T3)-R3(T3)*T3)
% Remember:
% cp = dh/dT, cv=du/dT

% -------------------------------------------
% Get some parameters
% -------------------------------------------
global psSimStruct
AFs = psSimStruct.psThermProp.AFs;

% -------------------------------------------
% Get thermochemical properties for old zones
% -------------------------------------------
outputs = zeros(1,9);
outputs([1 4]) = 1;

[old_h dummy1 dummy2 old_R] = psThermProp(p,[T1 ; T2],[Xgc1 ; Xgc2],outputs);

% -------------------------------
% Calculate masses for old zones
% -------------------------------
% Total mass of the zones
mTot_1  = p*V1/(old_R(1)*T1);
mTot_2  = p*V2/(old_R(2)*T2);
% Total mass of everything
mTot = mTot_1+mTot_2;
x1 = mTot_1/mTot;
x2 = mTot_2/mTot;

% -----------------------------
% Calculate new composition
% -----------------------------
Xgc = Xgc1*x1+Xgc2*x2;

% -----------------------------
% Calculate new temperature
% -----------------------------
if nargin < 13
  Tinit = (mTot_1*T1+mTot_2*T2)/(mTot);  % Initial guess
end


% Now calculate temperature using energy conservation
h_tot = (mTot_1*old_h(1)+...
 	 mTot_2*old_h(2))/mTot;
Vtot = V1+V2;

deltaT = Tinit;

while abs(deltaT) > abs(stopFrac*Tinit)
  outputs = [1 1 0 1 0 0 0 0 0];
  [new_h new_cp dummy2 new_R ]= psThermProp(p,Tinit,Xgc,outputs);
  deltaT  = -(new_h-h_tot)/(new_cp);
  Tinit = deltaT+Tinit;
 
  % pV = mRT =>
  p = mTot*new_R*Tinit/Vtot;
end

T = Tinit;
