%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

function [h cp cv R dRdp dRdT dudp dudX dRdX]=psThermProp(p,T,Xgc,outputs)
% [h cp cv R dRdp dRdT dudp dudX dRdX]=psThermProp(p,T,Xgc,outputs)
%  In: 
%    p       = Pressure        1x1
%    T       = Temperature     Nx1
%    Xgc     = Gas composition Nx4
%              [x_uf x_ua x_bf  x_ba]
%    outputs = Specifies wanted outputs 1x9
%  Out: 
%   h    = Nx1
%   cp   = Nx1
%   cv   = Nx1
%   R    = Nx1
%   dRdp = Nx1
%   dRdT = Nx1
%   dudp = Nx1
%   dudX = Nx4
%   dRdX = Nx4

persistent initialized
persistent nb mb pb nu mu
persistent CHEMPROPU
persistent CHEMPROPB
persistent R_f R_a
R_tilde = 8.3143;
if isempty(initialized)
  global psSimStruct
  CHEMPROPU=psSimStruct.psThermProp.chemPropU;
  CHEMPROPB=psSimStruct.psThermProp.chemPropB;

  initialized = 1;
  disp('Initializing psThermProp with values from CHEMPROPU/B')
  disp('These values are remembered by psThermProp and can only')
  disp('be cleared by ''clear psThermProp''')
  ps3Dinterpol(1,CHEMPROPB.T, CHEMPROPB.p, CHEMPROPB.xf,0 ,...
                 CHEMPROPB.hb    , ...
                 CHEMPROPB.cpb   , ...
                 CHEMPROPB.cvb   , ...
		 CHEMPROPB.Mb    , ...
		 CHEMPROPB.dRbdp , ...
		 CHEMPROPB.dRbdT , ...
                 CHEMPROPB.dubdp , ...
                 CHEMPROPB.dubdxf, ...
                 CHEMPROPB.dubdxa, ...
                 CHEMPROPB.dRbdxf, ...
                 CHEMPROPB.dRbdxa);
  ps1Dinterpol(1,CHEMPROPU.T,0 ,...
                 CHEMPROPU.ha,...
                 CHEMPROPU.cpa,...
                 CHEMPROPU.hf,...
                 CHEMPROPU.cpf);

%M->R;
R_f = R_tilde./CHEMPROPU.Mf;
R_a = R_tilde./CHEMPROPU.Ma;
end

% A zone is a mixing zone if T<1700 and a burned zone if T >=1700
% A burned zone can never have xuf > 0 or xua > 0. 
% burned	= find(T > 1700);
% unburned = find(T <= 1700);

% Cannot calc cv without R and cp for unburned so we calc them for both
% burned an unburned
if outputs(2)  
    outputs(4)= 1;
    outputs(1)= 1;
end

% Remember this during the implementation:
% Outputs:
%  h cp cv R dRdp dRdT dudp dudX dRdX
% Table order burned:
%  hb cpb cvb Mb dRbdp dRbdT dubdp  dubdxf dubdxa dRbdxf dRbdxa
% Table order unburned:
%  ha cpa Ma hf cpf Mf


xb = sum(Xgc(:,3:4),2);
xfu = Xgc(:,1);
xau = Xgc(:,2);


% Which of the burned props are needed
neededBurnedProps = zeros(1,11);
neededBurnedProps(1:7)= outputs(1:7);
if outputs(8)
    neededBurnedProps([8 9]) = 1;
end
if outputs(9)
    neededBurnedProps([10 11]) = 1;
end
intpIndexBurned = find(neededBurnedProps);

% Prepare a list wiht indexes where outputs may be found in the prop_b list
finalOutIndxsBurned(intpIndexBurned)=1:length(intpIndexBurned);

% Get the needed burned props
prop_b = zeros(length(T), length(intpIndexBurned));
for ii = 1:length(T)
    if any(Xgc(ii,3:4)) || any(neededBurnedProps(8:11)) 
        if xb(ii)
          xfb_local = Xgc(ii,3)./xb(ii);
          prop_b(ii,:) = ps3Dinterpol(ii, T(ii), p, xfb_local,intpIndexBurned);
          % Mb --> Rb
          if  outputs(4)
              prop_b(ii,finalOutIndxsBurned(4)) = ...
                  R_tilde./prop_b(ii,finalOutIndxsBurned(4));
          end
        else
          prop_b(ii,:) = 0; % WRONG!!
        end
    end
end


% Get the needed unburned props
prop_u = zeros(length(T), 4);
for ii = 1:length(T)
    if any([xfu(ii) xau(ii)])
        if T(ii) > 2000
            disp('T >= 2000 for mixing zone')
            %pause
        end    
        prop_u(ii,:) = ps1Dinterpol(ii, T(ii),1:4);
    else
        prop_u(ii,:) = 0; % WRONG!
    end        
end

% Remember this during the implementation:
% Outputs:
%  h cp cv R dRdp dRdT dudp dudX dRdX
% Table order burned:
%  hb cpb cvb Mb dRbdp dRbdT dubdp  dubdxf dubdxa dRbdxf dRbdxa
% Table order unburned:
%  ha cpa hf cpf

cp   = 0;
cv   = 0;
h    = 0;
R    = 0;
dRdp = 0;
dRdT = 0;
dudp = 0;
dudX = 0;
dRdX = 0;


if  outputs(1)
    h = prop_u(:,1).*xau+prop_u(:,3).*xfu+xb.*prop_b(:,finalOutIndxsBurned(1));
end
if  outputs(2)
    cp = prop_u(:,2).*xau+prop_u(:,4).*xfu+xb.*prop_b(:,finalOutIndxsBurned(2));
end
if  outputs(4)
    R = xfu*R_f+xau*R_a+xb.*prop_b(:,finalOutIndxsBurned(4));
end
if  outputs(3)
    % Mixing case -> cvu = cpu-Ru
    cvu = xau.*(prop_u(:,2)-R_a)+xfu.*(prop_u(:,4)-R_f);
    cv = cvu+xb.*prop_b(:,finalOutIndxsBurned(3));
end
if  outputs(5)
    % R = Ru*xu+Rb*xb => dRdp = dRudp*xu+dRbdp*xb = / dRudp=0 / = dRbdp*xb
    dRdp = prop_b(:,finalOutIndxsBurned(5)).*xb;
end
if  outputs(6)
    % R = Ru*xu+Rb*xb => dRdT = dRudT*xu+dRbdT*xb = / dRudT=0 / = dRbdT*xb
    dRdT = prop_b(:,finalOutIndxsBurned(6)).*xb;
end
if  outputs(7)
    % u = uu*xu+ub*xb => dudp = duudp*xu+dubdp*xb = / duudp=0 / = dubdp*xb
    dudp = prop_b(:,finalOutIndxsBurned(7)).*xb;
end
if  outputs(8)

    %    u = = ub*xb;
    %    dudxfb =  ub + xb*dubdxf
    ub = prop_b(:,finalOutIndxsBurned(1))-prop_b(:,finalOutIndxsBurned(4)).*T;
    duudxa = prop_u(:,1)-R_a*T;
    duudxf = prop_u(:,3)-R_f*T;
    dudX = [duudxf...
            duudxa...
            ub+prop_b(:,finalOutIndxsBurned(8))...
            ub+prop_b(:,finalOutIndxsBurned(9))];
end
if  outputs(9)
    %dRdX = [dRdxfu dRdxau dRbdxf dRbdxa]
    %R = xb.*prop_b(:,finalOutIndxsBurned(4));
    Rb=prop_b(:,finalOutIndxsBurned(4));
    dRdX = [repmat(R_f,length(T),1) ...
            repmat(R_a,length(T),1) ...
            Rb+prop_b(:,finalOutIndxsBurned(10)) ...
            Rb+prop_b(:,finalOutIndxsBurned(11))];
end


% ---------------------------------------------------------------------------
function res = ps1Dinterpol(zoneNr, x,outIndxs,varargin)
% Linear 1d-interpolation. Out of range are linarly
% extrapolated.
% Deklarera variabler som skall kommas ihåg mellan exekveringarna
persistent LOCAL_DATA
persistent LOCAL_X 
persistent LOCAL_DX
persistent LOCAL_XIDX 
persistent LAST_IDX;
% Kolla om initiering
if nargin > 3
  LOCAL_DATA = [];
  for i = 1:length(varargin),
      LOCAL_DATA(i, 1:length(varargin{i})) = varargin{i};
  end

  LOCAL_X = x;

  % Utnyttjar antagande om ekvidistanta data
  LOCAL_DX = diff(x);

  % Sätter grundvärden på indexpekare
  LOCAL_XIDX = 1;
  LAST_IDX   = [0 0];
  res = 1;
else
% Kolla om det är dags att producera utdata
  if (nargin == 3) 
    % Här ska jobbet göras

    % 1. Kolla gränser på indata
    x = max(x, min(LOCAL_X));
    x = min(x, max(LOCAL_X));

    % 2. Beräkna parameteriska koordinaten: r
    % 2.1 Först: Är alla zoner definerade?
    if (length(LOCAL_XIDX) <  zoneNr )
      LOCAL_XIDX( zoneNr ) = 1;
      LAST_IDX( zoneNr)   = 0;      
    end
    
    
    % 2.1 Beräkna först index i index-vektorerna med binärsökning.
    % 2.1.1 Beräkna sedan parametriska koordinater r,s och t
    
    % X
    ul = length(LOCAL_X);
    ll = 1;
    while ~((x >= LOCAL_X(LOCAL_XIDX( zoneNr )))  && ...
	    (x <= LOCAL_X(LOCAL_XIDX( zoneNr ) + 1)))
      if (x > LOCAL_X(LOCAL_XIDX( zoneNr ) + 1))
	ll = LOCAL_XIDX( zoneNr );
      else
	ul = LOCAL_XIDX( zoneNr );	
      end
      LOCAL_XIDX( zoneNr ) = max(ll + floor((ul - ll)/2), 1);
      LOCAL_XIDX( zoneNr ) = min(LOCAL_XIDX( zoneNr ), length(LOCAL_X) - 1);
    end
    r     = (x-LOCAL_X(LOCAL_XIDX( zoneNr )))/LOCAL_DX(LOCAL_XIDX( zoneNr ));

        
    % 3. Beräkna utdata
    % 3.1. Ta fram "hörnpunkter"
    if any(LAST_IDX( zoneNr )-LOCAL_XIDX( zoneNr ) )
        points = [LOCAL_DATA(outIndxs(:), LOCAL_XIDX( zoneNr )), ... % 0
                  LOCAL_DATA(outIndxs(:), LOCAL_XIDX( zoneNr ) + 1)];
    end
    W = [(1-r); r];

    res = (points*W)';
  else
    error('Felaktigt anrop');
  end
end
% ---------------------------------------------------------------------------
function res = ps3Dinterpol(zoneNr, x, y, z, outIndxs, varargin)
% Deklarera variabler som skall kommas ihåg mellan exekveringarna
persistent LOCAL_DATA
persistent LOCAL_X LOCAL_Y LOCAL_Z
persistent LOCAL_DX LOCAL_DY LOCAL_DZ
persistent LOCAL_XIDX LOCAL_YIDX LOCAL_ZIDX
persistent LAST_IDX;
persistent LAST_OUTINDXS;
% Kolla om initiering
if nargin > 5
  LOCAL_DATA = [];
  for i = 1:length(varargin),
    LOCAL_DATA(i, 1:size(varargin{i},1), 1:size(varargin{i},2), 1:size(varargin{i},3)) = varargin{i};
  end

  LOCAL_X = x;
  LOCAL_Y = y;
  LOCAL_Z = z;

  % Utnyttjar antagande om ekvidistanta data
  LOCAL_DX = diff(x);
  LOCAL_DY = diff(y); 
  LOCAL_DZ = diff(z); 

  % Sätter grundvärden på indexpekare
  LOCAL_XIDX = 1;
  LOCAL_YIDX = 1; 
  LOCAL_ZIDX = 1;
  LAST_IDX   = [0 0 0];
  res = 1;
else
% Kolla om det är dags att producera utdata
  if (nargin == 5) 
    % Här ska jobbet göras

    % 1. Kolla gränser på indata
    x = max(x, min(LOCAL_X));
    x = min(x, max(LOCAL_X));

    y = max(y, min(LOCAL_Y));
    y = min(y, max(LOCAL_Y));

    z = max(z, min(LOCAL_Z));
    z = min(z, max(LOCAL_Z));
    
    % 2. Beräkna parameteriska koordinater: r, s, t
    % 2.1 Först: Är alla zoner definerade?
    if (length(LOCAL_XIDX) <  zoneNr )
      LOCAL_XIDX( zoneNr ) = 1;
      LOCAL_YIDX( zoneNr ) = 1; 
      LOCAL_ZIDX( zoneNr ) = 1;
      LAST_IDX( zoneNr, : )   = [0 0 0];      
    end
    
    
    % 2.1 Beräkna först index i index-vektorerna med binärsökning.
    % 2.1.1 Beräkna sedan parametriska koordinater r,s och t


    ul = length(LOCAL_X);
    ll = 1;
    if LOCAL_XIDX( zoneNr ) < 1 
        LOCAL_XIDX( zoneNr )  = 1;
    end
    
    while ~((x >= LOCAL_X(LOCAL_XIDX( zoneNr )))  && ...
	    (x <= LOCAL_X(LOCAL_XIDX( zoneNr ) + 1)))
      if (x > LOCAL_X(LOCAL_XIDX( zoneNr ) + 1))
	ll = LOCAL_XIDX( zoneNr );
      else
	ul = LOCAL_XIDX( zoneNr );	
      end
      LOCAL_XIDX( zoneNr ) = max(ll + floor((ul - ll)/2), 1);
      LOCAL_XIDX( zoneNr ) = min(LOCAL_XIDX( zoneNr ), length(LOCAL_X) - 1);
    end
    r     = (x-LOCAL_X(LOCAL_XIDX( zoneNr )))/LOCAL_DX(LOCAL_XIDX( zoneNr ));

    if LOCAL_YIDX( zoneNr ) < 1 
        LOCAL_YIDX( zoneNr )  = 1;
    end

    ul = length(LOCAL_Y);
    ll = 1;
    while ~((y >= LOCAL_Y(LOCAL_YIDX( zoneNr )))  && (y <= LOCAL_Y(LOCAL_YIDX( zoneNr ) + ...
						  1)))
      if (y > LOCAL_Y(LOCAL_YIDX( zoneNr ) + 1))
	ll = LOCAL_YIDX( zoneNr );
      else
	ul = LOCAL_YIDX( zoneNr );	
      end
      LOCAL_YIDX( zoneNr ) = max(ll + floor((ul - ll)/2), 1);
      LOCAL_YIDX( zoneNr ) = min(LOCAL_YIDX( zoneNr ), length(LOCAL_Y) - 1);
    end
    s  = (y-LOCAL_Y(LOCAL_YIDX( zoneNr )))/LOCAL_DY(LOCAL_YIDX( zoneNr ));
    
    if LOCAL_ZIDX( zoneNr ) < 1 
        LOCAL_ZIDX( zoneNr )  = 1;
    end

    ul = length(LOCAL_Z);
    ll = 1;
    if length(LOCAL_Z) > 1
        while ~((z >= LOCAL_Z(LOCAL_ZIDX( zoneNr )))  && ...
                (z <= LOCAL_Z(LOCAL_ZIDX( zoneNr ) + 1)))
            if (z > LOCAL_Z(LOCAL_ZIDX( zoneNr ) + 1))
                ll = LOCAL_ZIDX( zoneNr );
            else
                ul = LOCAL_ZIDX( zoneNr );	
            end
            LOCAL_ZIDX( zoneNr ) = max(ll + floor((ul - ll)/2), 1);
            LOCAL_ZIDX( zoneNr ) = min(LOCAL_ZIDX( zoneNr ), length(LOCAL_Z) - 1);
        end
        t  = (z-LOCAL_Z(LOCAL_ZIDX( zoneNr )))/LOCAL_DZ(LOCAL_ZIDX( zoneNr ));
        
        % 3. Beräkna utdata
        % 3.1. Ta fram "hörnpunkter"
        if any(LAST_IDX( zoneNr ,:)-[LOCAL_XIDX( zoneNr ) LOCAL_YIDX( zoneNr ) LOCAL_ZIDX( zoneNr )])
            points = zeros(length(outIndxs), 8);
            for i = 1:length(outIndxs)
                points(i,:) = [LOCAL_DATA(outIndxs(i), LOCAL_XIDX( zoneNr ),     LOCAL_YIDX( zoneNr ),     LOCAL_ZIDX( zoneNr )), ... % 0
                               LOCAL_DATA(outIndxs(i), LOCAL_XIDX( zoneNr ) + 1, LOCAL_YIDX( zoneNr ),     LOCAL_ZIDX( zoneNr )), ... % 1
                               LOCAL_DATA(outIndxs(i), LOCAL_XIDX( zoneNr ),     LOCAL_YIDX( zoneNr ) + 1, LOCAL_ZIDX( zoneNr )), ... % 2
                               LOCAL_DATA(outIndxs(i), LOCAL_XIDX( zoneNr ) + 1, LOCAL_YIDX( zoneNr ) + 1, LOCAL_ZIDX( zoneNr )), ... % 3
                               LOCAL_DATA(outIndxs(i), LOCAL_XIDX( zoneNr ),     LOCAL_YIDX( zoneNr ),     LOCAL_ZIDX( zoneNr ) + 1), ... % 4
                               LOCAL_DATA(outIndxs(i), LOCAL_XIDX( zoneNr ) + 1, LOCAL_YIDX( zoneNr ),     LOCAL_ZIDX( zoneNr ) + 1), ... % 5
                               LOCAL_DATA(outIndxs(i), LOCAL_XIDX( zoneNr ),     LOCAL_YIDX( zoneNr ) + 1, LOCAL_ZIDX( zoneNr ) + 1), ... % 6
                               LOCAL_DATA(outIndxs(i), LOCAL_XIDX( zoneNr ) + 1, LOCAL_YIDX( zoneNr ) + 1, LOCAL_ZIDX( zoneNr ) + 1), ... % 7
                              ];
            end
        end
        W = [(1-r)*(1-s)*(1-t); ...
             r*(1-s)*(1-t); ...
             (1-r)*s*(1-t); ...
             r*s*(1-t); ...
             (1-r)*(1-s)*t; ...
             r*(1-s)*t;...
             (1-r)*s*t; ...
             r*s*t];
        
        res = (points*W)';
    else
        if any(LAST_IDX( zoneNr,1:2 )-[LOCAL_XIDX( zoneNr ) LOCAL_YIDX( zoneNr ) ])
            points = zeros(length(outIndxs), 4);
            for i = 1:length(outIndxs)
                points(i,:) = [LOCAL_DATA(outIndxs(i), LOCAL_XIDX( zoneNr ),     LOCAL_YIDX( zoneNr ),     LOCAL_ZIDX( zoneNr )), ... % 0
                               LOCAL_DATA(outIndxs(i), LOCAL_XIDX( zoneNr ) + 1, LOCAL_YIDX( zoneNr ),     LOCAL_ZIDX( zoneNr )), ... % 1
                               LOCAL_DATA(outIndxs(i), LOCAL_XIDX( zoneNr ),     LOCAL_YIDX( zoneNr ) + 1, LOCAL_ZIDX( zoneNr )), ... % 2
                               LOCAL_DATA(outIndxs(i), LOCAL_XIDX( zoneNr ) + 1, LOCAL_YIDX( zoneNr ) + 1, LOCAL_ZIDX( zoneNr )), ... % 3
                              ];
            end
        end
        W = [(1-r)*(1-s); ...
             r*(1-s); ...
             (1-r)*s; ...
             r*s];
        
        res = (points*W)';
    end

    else
    error('Felaktigt anrop');
  end
end


%% Validity-test
if 0
    chempropb = psSimStruct.psThermProp.chemPropB;
    pInd= 15;
    p = chempropb.p(pInd);
    Tmin = min(chempropb.T);
    Tmax = max(chempropb.T);
    xfmin = min(chempropb.xf);
    xfmax = max(chempropb.xf);
    xf_vec = xfmin:(xfmax-xfmin)/51:xfmax;
    T_vec = Tmin:(Tmax-Tmin)/51:Tmax;
    prop_b= zeros(length(xf_vec),length(T_vec),11);
     for kk = 1:11
       outputs = zeros(1,11);
       outputs(kk) = 1;
       for ii = 1:length(xf_vec)
         xf_i = xf_vec(ii);
         for jj = 1:length(T_vec)
           T_j= T_vec(jj);
           [prop_b(ii,jj,kk)]=psThermProp(p,T_j,[0 0 xf_i  1-xf_i],outputs);
         end        
       end
     end
    fNames = {'cpb','cvb','hb','Mb','dubdp','dRbdp','dRbdT', ...
              'dubdxf','dRbdxf','dubdxa','dRbdxa'}; 
    
    for ii = 1:length(fNames)
       figure
       mesh(chempropb.xf,chempropb.T,squeeze(chempropb.(fNames{ii})(:,pInd,:)))
       hold on
       mesh(xf_vec,T_vec,prop_b(:,:,ii)')
       title(fNames{ii})
       xlabel('x_f [-]')
       ylabel('T [K]')       
    end


    prop_b2= zeros(length(xf_vec),length(T_vec),3);
     outputs = zeros(1,11);
     outIndxs= [1 3 5];
     outputs([1 3 5]) = 1;
     for ii = 1:length(xf_vec)
         xf_i = xf_vec(ii);
         for jj = 1:length(T_vec)
             T_j= T_vec(jj);
             [prop_b2(ii,jj,:)]=psThermProp(p,T_j,[0 0 xf_i  1-xf_i],outputs);
        end        
     end

     for ii = 1:length(outIndxs)
         figure
         mesh(chempropb.xf,chempropb.T,squeeze(chempropb.(fNames{outIndxs(ii)})(:,pInd,:)))
         hold on
         mesh(xf_vec,T_vec,prop_b2(:,:,ii)')
         title(fNames{ii})
         xlabel('x_f [-]')
         ylabel('T [K]')       
    end

    
    figure;
    plot(xf_vec,prop_b(:,1,end))
    xlabel('x_f [-]')
    ylabel('dRb/dxa')
    grid on

    % Unburned
    phi = 1;
    AFs = 14.36;
    xf_i = (phi/AFs)/((1+phi/AFs));
    T_j = 1500;
    p  = 1e5;
    outputs = ones(1,11);
    [h cp cv R dRdp dRdT dudp dudX dRdX] = psThermProp(p,T_j,[xf_i  1-xf_i 0 0],outputs) 
end