%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

function outValue =  psStartUp(method,arg1,arg2,arg3)
% ------------------------------>psStartUp<----------------------------------
% psStartup is used for starting psPack and controlling its
% behaviour. It has different methods that are called using 
% > psStartUp('methodName',arg1,arg2,arg3)
%  
% The different methods are described in the code but mostly only
% gui functions are needed.
%
% A normal user would simply do the following
% * run "psStartUp();"
% * Use  the gui to set parameters
% * Run "global psSimStruct"
% * Run "sim('psModel')"
%
% ------------------------------>psStartUp<----------------------------------
%  
% Methods:
%  startUp                : Initializes the program-controlstructs
%                           from scratch. 
%  setMainWindowVisible   : Hides/Unhides mainwindow
%  initControlStruct      : Set up the most basic controlstruct
%                           data. Then automatically run
%                           'updateControlStruct'
%  initSimStruct          : Basic initialization of the psSimStruct.
%  updateControlStruct    : Reads controlinformation that is needed to 
%                           correctly draw window elements
%                           etc. This function checks the local
%                           directory for psPack-stuff and sets the
%                           paths accordingly.
%  loadSimStruct          : Sets the psSimStruct variable to some
%                           reasonable values-according to the
%                           psSimStruct in arg1, but with care
%                           taken to the actual psPack-dist-directory.
%  createMainWindow       : Create the mainWindow. Destroy the old
%                           window and create a new one if there
%                           happens to be one already.
%  updateMainWindow       : Sets mainWindow-elements to actual values
%                           from controlStruct. In the meantime
%                           this function just calls 'createMainWindow'
%  psListBoxClBck         : Executes operations necesarry after
%                           chage of listbox values.
%  psInfoClBck            : Executes operations necessary when a
%                           information-button ha been pressed.
%  psInitClBck            : Executes necessary operations after a
%                           crete-fcn or init-button has been pressed
%  psClearClBck           : Clears away local copy of actual
%                           psFuncName- functionality.
%  psAboutClbck           : Displays about-popup-information
%  psNewSimulClbck        : Executes operations necessary when a
%                           new simulation is wanted.  
%  psChangeDirClbck       : Sets psControlStruct.currentSimDir and
%                           changes directory. 
%  psCloseClbck           : Shuts psPack down and clears
%                           local/persistent and global variables  
%  psDocClbck             : Ececutes documentation-operations. 
%                           Without in-arguments a browser is
%                           pointed to the main documentation-page.
%  psLoadSimulClbck       : Popups a 'new-simulation'
%                           question-dialog. If 'stealth' mode is
%                           wanted loadSimStruct can be used
%                           instead 
%  psOpenSimulClbck       : Popups a 'new-simulation'
%                           question-dialog. If 'stealth' mode is
%                           wanted loadSimStruct can be used
%                           instead. OpenSimul also changes directory
%  psSaveSimulClbck       : Saves the current simulation to file.
%  displayMessage         : Function to display a message. This could
%                           be any message, but this function is
%                           usually called by psInfoClbck 
%  getControlStruct       : Returns the current psControlStruct,
%                           mainly needed for debuging purposes
% 
  
% ===========================================================================
%  Global initialization
% ===========================================================================

% Some variables are initialized to a sort of basic intialization state
  outValue = 0;    
  if evalin('base','isempty(who(''psSimStruct''))')
      disp('Creating globals: ''psSimStruct, psSimOut''')
      evalin('base','global psSimStruct')
      evalin('base','global psSimOut')
      %else
  %    disp('psSimStruct exists')
  end
  global psDebugLevel
  global psSimStruct;
  persistent psControlStruct;
 
  if isempty(psDebugLevel)
      psDebugLevel = 0;
  end
  
  % -----------------------------------
  % Prevent data loss due to clear
  % -----------------------------------
  mlock
  % If the user clears psSimStruct by mistake a copy is always
  % stored here. This is done in the "finish up" part down below
  persistent psSimStructCopy;
  if isempty(psSimStruct)
    psSimStruct = psSimStructCopy;
  end
  
  % Do we need to call restart.
  if nargin == 0
    method = 'startUp'; 
  end

  if isempty(psControlStruct) && ...
        ~strcmp(method,'psCloseClbck') && ...
        ~strcmp(method,'initControlStruct');
    method = 'startUp';
  end

  % Some structure defining stuff
  psFuncNames = {'psMeas';...
                 'psVibe';...
                 'psValve';...
                 'psHeat';...
                 'psModel';...
                 'psKernel';...
                 'psGeometry';...
                 'psThermProp';...
                 'psSimPar';};
  psFuncDirs  = {'Meas';...
                 'Vibe';...
                 'Valve';...
                 'Heat';...
                 'Models';...
                 'Kernel';...
                 'Geometry';...
                 'ThermProp';...
                 'SimPar'};
  
  % Define some look-defining variables  
  submoduleXSize = 600-8;
  commandXSize   = 150-4;
  mainFigXSize   = submoduleXSize + commandXSize + 12;
  mainFigYSize   = 580;
  textBoxYSize   = 20; % Pixels
  textBoxXSize   = 90; % Pixels
  textBoxXDiff   = 30; % Pixels
  textBoxXStart  = 20;
  textBoxYStart  =  mainFigYSize-60;

  listBoxYSize = 20; % Pixels
  listBoxXSize = 120; % Pixels
  listBoxXDiff = textBoxXDiff;
  listBoxXStart = textBoxXStart+textBoxXSize+10;
  listBoxYStart = textBoxYStart+4;

  clearButtonYSize = 20; % Pixels
  clearButtonXSize = 90; % Pixels
  clearButtonXDiff = textBoxXDiff;
  clearButtonXStart = listBoxXStart+listBoxXSize+20;
  clearButtonYStart = textBoxYStart;

  infoButtonYSize = 20; % Pixels
  infoButtonXSize = 90; % Pixels
  infoButtonXDiff = textBoxXDiff;
  infoButtonXStart = clearButtonXStart+clearButtonXSize+20;
  infoButtonYStart = textBoxYStart;

  initButtonYSize = 20; % Pixels
  initButtonXSize = 90; % Pixels
  initButtonXDiff = textBoxXDiff;
  initButtonXStart = infoButtonXStart+infoButtonXSize+20;
  initButtonYStart = textBoxYStart;

  submoduleYDiff = 40; % Pixels
  
  psBgColor_light = [1.0 1.0 1.0]*0.8;
  psBgColor_dark = psBgColor_light*0.7;
  psBgColor_inactive = psBgColor_light*0.9;
  psFgColor_inactive = [0.5 0.5 0.5];
  psBgColor_ok = [0.0 1.0 0.0];
  psBgColor_error = [1.0 0.0 0.0];
  psBgColor_error_2 = [1.0 0.7 0.7];
  psBgColor_ok2   = [1.0 1.0 0.0];

  
  psCommandButtonYSize   = 20; % Pixels
  psCommandButtonXSize   = 90; % Pixels
  psCommandButtonXDiff   = 30; % Pixels
  psCommandButtonXStart  = 20;
  psCommandButtonYStart  =  mainFigYSize-60;

  commandButtonYDiff = 40;


% ===========================================================================
% Methods
% ===========================================================================
switch method
  % -------------------------------------------------------------------------
  % Method      : startUp
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <-> hardRestart (If hardRestart = 0 then don't RE-start)
  % -------------------------------------------------------------------------
  % Description : Initializes the program-controlstructs
  %               from scratch. If there already exists a
  %               psControlStruct some questions are asked.
  % -------------------------------------------------------------------------
case 'startUp'
  if nargin >=2
      hardRestart = arg1;
  else
      hardRestart = 1;
  end
  if ~isempty(psControlStruct) 
    if hardRestart
        button = questdlg(['This will restart psPack' ...
                           ' and end the current instance. Are you sure?'], ...
                           'psPack 1.0 (Warning)','Yes','No','No');
        if strcmp(button,'Yes')
          if isfield(psControlStruct,'mainFigure') && ...
                 isfield(psControlStruct.mainFigure,'handle')
            psStartUp('psCloseClbck',psControlStruct.mainFigure.handle,1);
          end
          reallyDoStartUp = 1;
        else
          reallyDoStartUp = 0;
          psStartUp('updateControlStruct');
          psStartUp('updateMainWindow');
        end
    else
          reallyDoStartUp = 0;
    end
  else
    reallyDoStartUp = 1;
  end
  if reallyDoStartUp
    psControlStruct = 1;  
    %    button = questdlg('Do you want to load a previus simulation', ...
    %                      'psPack','Yes','No','No');
    %    if strcmp(button,'Yes')
    %      simLoadStatus = psStartUp('psLoadSimulClbck');
    %      if simLoadStatus == -1
    %        psStartUp('initControlStruct');
    %        psStartUp('initSimStruct');
    %        psStartUp('updateControlStruct');
    %        psStartUp('updateMainWindow');
    %      end
    %    else
    psStartUp('initControlStruct');
    psStartUp('initSimStruct');
    psStartUp('updateControlStruct');
    psStartUp('updateMainWindow');
    % end  
  end
  % -------------------------------------------------------------------------
  % End Method: startUp
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : setMainWindowVisible
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : enable/disable drawing of mainwindow
  % -------------------------------------------------------------------------
 case 'setMainWindowVisible'
  psControlStruct.mainFigure.visible = arg1;
  if psControlStruct.mainFigure.handle
      if  psControlStruct.mainFigure.visible
          set(psControlStruct.mainFigure.handle,'Visible','on')
      else
          set(psControlStruct.mainFigure.handle,'Visible','off')
      end
  end
  
  % -------------------------------------------------------------------------
  % End Method: startUp
  % -------------------------------------------------------------------------
   
  % -------------------------------------------------------------------------
  % Method      : initControlStruct 
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Basic initialization of the psControlStruct.
  % -------------------------------------------------------------------------
 case 'initControlStruct'
  psControlStruct = [];
  psControlStruct.mainFigure.handle = 0;
  psControlStruct.mainFigure.visible = 1;
  psControlStruct.initialized = 1;
  psControlStruct.currentSimDir = pwd;
  tmpStr1 = which('psStartUp');
  tmpStr2 = dir(tmpStr1);
  tmpStr2 = tmpStr2.name;
  tmpIdx  = strfind(tmpStr1,tmpStr2);
  psControlStruct.distDir = tmpStr1(1:(tmpIdx-2));
  addpath(psControlStruct.distDir);
  while min(psControlStruct.currentSimDir == 0) || ...
     (~isempty(strfind(psControlStruct.currentSimDir,psControlStruct.distDir)) ...
      && (strfind(psControlStruct.currentSimDir,psControlStruct.distDir) == 1))
    button = warndlg(['You have started psPack from the' ...
                      ' distribution directory.' ...
                      ' Please select another directory.'],...
                     'Warning (initialization)');
    uiwait(button);
    psControlStruct.currentSimDir =  uigetdir;
    if  psControlStruct.currentSimDir
      cd(psControlStruct.currentSimDir);
    end
  end
  
  % Needed if the user changes directory.
  addpath(pwd); 
  
  for tmpFuncIdx = 1:length(psFuncNames);    
    psFuncName = psFuncNames{tmpFuncIdx};
    psControlStruct.(psFuncName).activeElement = 0;
    psControlStruct.(psFuncName).funcDir=psFuncDirs{tmpFuncIdx};
  end
  % -------------------------------------------------------------------------
  % End Method: initControlStruct
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : initSimStruct 
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Basic initialization of the psSimStruct variable.
  % -------------------------------------------------------------------------
 case 'initSimStruct'
  psSimStruct = [];
  for tmpFuncIdx = 1:length(psFuncNames);    
    psFuncName = psFuncNames{tmpFuncIdx};
    psSimStruct.(psFuncName) = [];
    psSimStruct.(psFuncName).varInfo.initialized = 0;
  end
  % -------------------------------------------------------------------------
  % End Method: initSimStruct
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : updateControlStruct 
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <-> psFuncName
  % -------------------------------------------------------------------------
  % Description : Updates the information in psControlStruct
  %               according to current filesystem-layout. If arg1
  %               is specified only the 'psFuncName' in arg1 is updated.
  % -------------------------------------------------------------------------
 case 'updateControlStruct'
  
  upDateWhichModules = 1:length(psFuncNames);
  if nargin >= 2
    upDateWhichModules = find(strcmp(arg1,psFuncNames));
  end    
  for tmpFuncIdx = upDateWhichModules
    % Collect information about subdirectories etc. to find out what
    % methods are availeble
    % Search for [psFuncName 'Init.m']    
    psFuncName = psFuncNames{tmpFuncIdx};
    psFuncDir = psFuncDirs{tmpFuncIdx};
    psLookforFcn = 'Init.m';
    fullDirName = [psControlStruct.distDir filesep ...
                   psControlStruct.(psFuncName).funcDir];
    dirContent = dir(fullDirName); 
    dirNames = [];
    dirPaths = [];
    numDirs = 0;
    for k = 1:length(dirContent)
      tmpDirPath = [fullDirName filesep dirContent(k).name];
      [tmpDirPath filesep psFuncName psLookforFcn];
      if ~isempty(dir([tmpDirPath filesep psFuncName psLookforFcn]))
        numDirs = numDirs+1;
        dirPaths{numDirs} = tmpDirPath;   
        dirNames{numDirs} = dirContent(k).name;
      end
    end
    
    activeElement = 1;
    localName = dir([psControlStruct.currentSimDir filesep psFuncName ...
                     psLookforFcn]); 
    localExists = 0;
    
    if ~isempty(localName) && strcmp(localName.name,[psFuncName ...
                          psLookforFcn]) 
      localExists = 1;
      dirNames{numDirs+1} = 'Local';
      dirPaths{numDirs+1} = psControlStruct.currentSimDir;
      activeElement = numDirs+1;
    end
    
    psControlStruct.(psFuncName).dirNames = dirNames;
    psControlStruct.(psFuncName).dirPaths = dirPaths;
    
    %      disp(sprintf('Active element: %f ',...
    %                   psControlStruct.(psFuncName).activeElement))
    if psControlStruct.(psFuncName).activeElement > length(dirNames) ...
          || ~psControlStruct.(psFuncName).activeElement 
      %disp(sprintf('Changed Active element to: %f ',activeElement))
      psControlStruct.(psFuncName).activeElement = activeElement;
    end 
    
    psControlStruct.(psFuncName).localExists = localExists;

    % Kontrollera att anv�ndaren inte g�r in i psDist katalogen
    if strfind(pwd,psControlStruct.distDir)
      h = warndlg(['You have changed directory to a ', ...
                   'working-directory that could contain functions ',...
                   'that interfears with psPack. Your current ',...
                   'directory will be changed back'],'!! Warning !!');
      set(h,'WindowStyle','modal');
      uiwait(h);
      cd(psControlStruct.currentSimDir);
    end               

    % ---------------------------------------------------------------
    % L�gg till r�tt 'psFuncName' katalog i pathen. Ta bort alla andra
    % kataloger och underkataloger f�rst f�r att vara s�ker. Ta
    % dock inte bort eventuella underkataloger till den aktuella
    % d� de kan vara tillagda av undermodulen sj�lv.
    % ---------------------------------------------------------------
    
    % Lite variabler som beh�ver vara = 0 n�r vi b�rjar
    tmpIsLocal = 0;
    tmpIsEmpty = 0;
    % Ta reda p� vilken katalog som kommer med det aktuella valet
    % av undermodul.
    tmpActiveElement = psControlStruct.(psFuncName).activeElement;
    tmpActiveName = ...
        psControlStruct.(psFuncName).dirNames{tmpActiveElement};
    tmpActiveDir = [psControlStruct.distDir filesep ...
                    psFuncDir filesep tmpActiveName];
    % Kolla hur m�nga psXXInit det finns i pathen och ta bort
    % alla utom active element om den redan finns.      
    tmpWhich  = which([psFuncName 'Init'],'-all');
    for fn_nbr = 1:length(tmpWhich)
      % Plocka ut katalogerna en och en
      tmpStr1 = tmpWhich{fn_nbr};
      % Ta reda p� filnamnet s� att det kan strippas bort
      tmpStr2 = dir(tmpStr1);
      if ~isempty(tmpStr2) % S�kerhets�tg�rd
                           % Plocka ut filnamnet
        tmpStr2 = tmpStr2.name;
        % Plocka bort filnamnsdelen, kvar ska nu vara
        % katalognamnet med s�kv�g
        tmpIdx  = strfind(tmpStr1,tmpStr2);
        tmpDir = tmpStr1(1:(tmpIdx-2));
        
        % Ta reda p� om det �r den katalogen som h�r i hop med
        % det aktuella valet som vi hittat
        isActiveDir = strcmp(lower(tmpDir), lower(tmpActiveDir));
        
        if ~strcmp(lower(tmpDir),lower(psControlStruct.currentSimDir)) &&...
              ~isempty(strfind(lower(path),lower(tmpDir))) && ~isActiveDir
          fullPath = path;
          pathIdxs = strfind(fullPath,tmpDir);
          for tmpIdx = 1:length(pathIdxs)
            pathIdxs2 = min(pathIdxs(tmpIdx)-2+...
                            strfind(fullPath(pathIdxs(tmpIdx):end),pathsep));
            thePath = fullPath(pathIdxs(tmpIdx):pathIdxs2);
            rmpath(thePath);
            %disp(['Removing from path: ' thePath ]);
          end            
        end            
      end
    end    
    
    if ~psControlStruct.(psFuncName).localExists
      addpath(psControlStruct.(psFuncName).dirPaths{psControlStruct ...
                          .(psFuncName).activeElement});
      %disp(['Adding to path: ' ...
      %      psControlStruct.(psFuncName).dirPaths{psControlStruct ...
      %                    .(psFuncName).activeElement}]);
    end  
    
    % Check what type the init-function has.
    executable = [psFuncName 'Init(''checkType'')'];
    initType =   eval(executable);
    
    if strcmp(initType,'createLocal')
      psControlStruct.(psFuncName).createLocal = 1;
    else 
      psControlStruct.(psFuncName).createLocal = 0;
    end  
    
    if strcmp(initType,'localFile')
      psControlStruct.(psFuncName).([psFuncName 'Files']) = [];   
      if ~isempty(dir([psFuncName 'Files.mat']))
        tmpFiles = load([psFuncName 'Files']);
        psControlStruct.(psFuncName).([psFuncName 'Files']) = ...
            tmpFiles.([psFuncName 'Files']);
      end
    else
      psControlStruct.(psFuncName).([psFuncName 'Files']) = [];
    end           
  end
  
  % ---------------------------------------------------------------
  % Kolla vilka addons som finns i psDist/Addons katalogen och 
  % l�gg till de katalogerna i pathen
  % ---------------------------------------------------------------
  fullDirName = [psControlStruct.distDir filesep '..' filesep ...
                 'Addons'];
  dirContent = dir(fullDirName); 
  numAddons = 0;
  addonDirs = {};
  for k = 1:length(dirContent)
    if psDebugLevel >= 2
        dirContent(k)
    end
    if dirContent(k).isdir
      if ~((strcmp(dirContent(k).name,'.') || strcmp(dirContent(k).name,'..')))
        numAddons = numAddons+1;
        addonDirs{k} = dirContent(k).name;
        addpath([fullDirName filesep dirContent(k).name])
        if psDebugLevel >= 1
            disp(['Adding path to addon: ' addonDirs{k} ])
        end
      end
    end      
  end
  
  % -------------------------------------------------------------------------
  % End Method: updateControlStruct
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : loadSimStruct
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- newSimStruct
  % -------------------------------------------------------------------------
  % Description : A new psSimStruct is creted on the basis of
  %               newSimStruct. If a 'newSimStruct' refers to parts
  %               that does not exist in current path then these
  %               are left uninitialized.  
  % -------------------------------------------------------------------------
 case 'loadSimStruct'
  newSimStruct = arg1;
  for tmpFuncIdx = 1:length(psFuncNames);
    psFuncName = psFuncNames{tmpFuncIdx};
    if isfield(newSimStruct,psFuncName) && ...
            isfield(newSimStruct.(psFuncName),'varInfo') && ...
            length(fieldnames(newSimStruct.(psFuncName).varInfo)) > 1
      tmpMatchingIdx = 0;
      % Check if the origin of the loaded psFuncName part still
      % exists and if it does check what idx is has at the moment.
      tmpNumDirs  = length(psControlStruct.(psFuncName).dirNames);
      theDirNames = psControlStruct.(psFuncName).dirNames;
      theDirNames{tmpNumDirs+1} = 'Local';
      for tmpIdx = 1:tmpNumDirs
        if strcmp(psControlStruct.(psFuncName).dirNames(tmpIdx),...
                  {newSimStruct.(psFuncName).varInfo.origin})
          tmpMatchingIdx = tmpIdx;
        end
      end
      % If the origin of the loaded struct submodule matched some
      % dirName then make this the actual choice and set the variables.
      % The perform any special operations needed
      if tmpMatchingIdx
        psSimStruct.(psFuncName) = newSimStruct.(psFuncName);
        psControlStruct.(psFuncName).activeElement = tmpMatchingIdx;
        % Any paths in varInfo.specialInfo??
        if isfield(psSimStruct.(psFuncName).varInfo,'specialInfo')
         specialInfo=fieldnames(psSimStruct.(psFuncName).varInfo.specialInfo);
         for ii = 1:length(specialInfo)
             if ~isempty(strfind(lower(specialInfo{ii}),'path'))
             addpath([psControlStruct.distDir filesep psSimStruct.(psFuncName).varInfo.specialInfo.(specialInfo{ii})]);
             end         
         end
        end
        psStartUp('updateControlStruct');
        % Do we need to intialize, if so do it
        if ~psSimStruct.(psFuncName).varInfo.initialized
            workingDir = psControlStruct.currentSimDir;
            executable = [psFuncName 'Init(''initSavedSimulation'',''' workingDir ''',1,' ...
                          ' psSimStruct.(psFuncName))'];
            eval(executable);
        end
      end      
    end
  end
  % -------------------------------------------------------------------------
  % End Method: loadSimStruct
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : updateMainWindow 
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : This function should update the mainwindow with
  %               the most current information in
  %               psControlStruct. If no window exists
  %               'createMainWindow' should be called instead.
  % -------------------------------------------------------------------------
 case 'updateMainWindow'
   if psControlStruct.mainFigure.visible
       psStartUp('createMainWindow'); % For now
   end
  % -------------------------------------------------------------------------
  % End Method: updateMainWindow
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : createMainWindow 
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Create the mainWindow. Destroy the old
  %               window and create a new one if there
  %               happens to be one already.
  % -------------------------------------------------------------------------
 case 'createMainWindow'
  rehash pathreset
  psSimStruct=psCalcDepVars(psSimStruct);
  % Start Drawing  the "Main window"
  if ~psControlStruct.mainFigure.handle
    psControlStruct.mainFigure.handle = figure;
  end
  
  mainFigHandle = psControlStruct.mainFigure.handle;
  figure(psControlStruct.mainFigure.handle)
  clf;
  
  mainFigPos = get(mainFigHandle,'Position');
  if mainFigXSize ~= mainFigPos(4)
    mainFigPos(2) = mainFigPos(2)+ mainFigPos(4) - mainFigYSize;
  end

  set(psControlStruct.mainFigure.handle,'MenuBar','none',...
                    'Color',psBgColor_light,'Position',...
                    [mainFigPos(1) mainFigPos(2)...
                     mainFigXSize mainFigYSize], ...
                    'NumberTitle','off','Name','psPack (mainWindow)',...
                    'Resize','off','SelectionHighlight','off',...
                    'CloseRequestFcn',['psStartUp(''psCloseClbck'',' ...
                      num2str(mainFigHandle) ');']); 
  
  titleText = uicontrol(mainFigHandle,'style','text', ...
                        'String','-^- psPack -^-','FontSize',32,...
                        'BackgroundColor',psBgColor_light, ...
                        'Position',[0 mainFigYSize-50 mainFigXSize 50], ...
                        'SelectionHighlight','off',...
                        'TooltipString','psPack - 1.0 (About)');
  
  tmpFramePos = [4 ...
                 textBoxYStart-submoduleYDiff*(length(psFuncNames)+1) ...
                 submoduleXSize ...
                 submoduleYDiff*(length(psFuncNames)+1)] ;

  matlabVersion = version;
  if eval(matlabVersion(1))>6
    tmpFrameHdl = ...
        uipanel(mainFigHandle,'Title','Sub modules:','Unit',...
                'pixels','Position', ...
                tmpFramePos,'SelectionHighlight',...
                'off','BackgroundColor', ...
                psBgColor_light);

    tmpFramePos = 0;
  else
    tmpFrameHdl = mainFigHandle;
  end
  
  % Draw the selection part
  for tmpFuncIdx = 1:length(psFuncNames);    
    psFuncName = psFuncNames{tmpFuncIdx};
    
    % Draw the (psFuncName) textbox
    tmpTextBoxPos = [textBoxXStart submoduleYDiff* ...
                     (tmpFuncIdx-0.5)-1 textBoxXSize textBoxYSize]; 
    tmpTextBoxPos = tmpTextBoxPos +tmpFramePos.*[1 1 0 0];
    tmpTextBox = uicontrol(tmpFrameHdl,'style','text','String', ...
                           [psFuncName ':'],'Position', ...
                           tmpTextBoxPos,'BackgroundColor', ...
                           psBgColor_light,'SelectionHighlight','off',...
                           'HorizontalAlignment','right');
    
    
    % Draw the selection-thingy for 'psFuncName'
    dirNames = psControlStruct.(psFuncName).dirNames;
    activeElement = psControlStruct.(psFuncName).activeElement;  
    localExists = psControlStruct.(psFuncName).localExists;
    createLocal = psControlStruct.(psFuncName).createLocal;
    
    if localExists
      clearEnabledStr = 'on';
      if activeElement == length(dirNames) % The local is chosen
        tmpColor = psBgColor_ok;
      else    
        tmpColor = psBgColor_error;
      end        
    else
      clearEnabledStr = 'off';
      tmpColor = psBgColor_ok;      
    end
    
    if createLocal
      tmpColor = psBgColor_ok2;
    end
    
    toolTipString = ['Click to select the ' psFuncName...
                     ' sub-module you want.'] ;
    listBoxPos = [listBoxXStart submoduleYDiff* ...
                  (tmpFuncIdx-0.5) listBoxXSize listBoxYSize]; 
    listBoxPos = listBoxPos + tmpFramePos.*[1 1 0 0];
    listBox = uicontrol(tmpFrameHdl,'style','Popup Menu', ...
                        'String',dirNames,'Callback', ...
                     ['psStartUp(''psListBoxClBck'',''' psFuncName ''');'],...
                        'Position',listBoxPos, ...
                        'BackgroundColor',tmpColor,'Value', ...
                        activeElement,'SelectionHighlight','off', ...
                        'TooltipString',toolTipString); 
    
    psControlStruct.(psFuncName).listBox = listBox;
    
    % Draw  clear box.
    clearButtonPos = [clearButtonXStart  ...
                      submoduleYDiff*(tmpFuncIdx-0.5) clearButtonXSize ...
                      clearButtonYSize];  
    clearButtonPos = clearButtonPos + tmpFramePos.*[1 1 0 0];
    toolTipString = ['Click to move away local version of ' psFuncName ...
                     ' sub-module.']; 
    clearButton = uicontrol(tmpFrameHdl,'style','pushbutton', ...
                            'String','Clear Local','Callback', ...
                        ['psStartUp(''psClearClBck'',''' psFuncName ''');'],...
                            'Position',clearButtonPos, ...
                            'BackgroundColor',psBgColor_dark,'Value', ...
                            activeElement,'Enable',clearEnabledStr, ...
                            'SelectionHighlight','off', ...
                            'TooltipString',toolTipString); 
    
    infoButtonPos = [infoButtonXStart  ...
                     submoduleYDiff*(tmpFuncIdx-0.5) infoButtonXSize ...
                     infoButtonYSize];    
    infoButtonPos = infoButtonPos + tmpFramePos.*[1 1 0 0];
    toolTipString = ['Click to get information about the active choice  ' ...
                     'of ' psFuncName ' sub-module.'] ;
    infoButton = uicontrol(tmpFrameHdl,'style','pushbutton', ...
                           'String','Information','Callback', ...
                        ['psStartUp(''psInfoClBck'',''' psFuncName ''');'],...
                           'Position',infoButtonPos, ...
                           'BackgroundColor',psBgColor_dark, ...
                           'SelectionHighlight','off', ...
                           'TooltipString',toolTipString); 
    
    initButtonPos = [initButtonXStart  ...
                     submoduleYDiff*(tmpFuncIdx-0.5) initButtonXSize ...
                     initButtonYSize];   
    
    initButtonPos = initButtonPos + tmpFramePos.*[1 1 0 0];
    if localExists
      if activeElement == length(dirNames) % The local is chosen
        initEnabledStr = 'on';
      else    
        initEnabledStr = 'off';
        notEnabledReason = 'localExists';
      end        
    else
      initEnabledStr = 'on';
    end
    % Check basic prequisities for sub-model to be able to be initialized
    if strcmp(initEnabledStr,'on') 
        if psDebugLevel >= 1
            disp(['Sub-model ''' psFuncName ...
                  ''' might have prequisities, checking ...'])
        end        
        psFuncPrecData = eval([ psFuncName 'Init(''checkPreq'')']);
        if isstruct(psFuncPrecData) && psFuncPrecData.havePreqs
          if psDebugLevel >= 1
              disp('...  Yes, there are prequisities, checking them ...')
          end
          if ~psFuncPrecData.preq 
              initEnabledStr = 'off';
              notEnabledReason = psFuncPrecData.reason;
          end
        else
          if psDebugLevel >= 1
              disp('...  No, there are no prequisities')
              
          end        
        end
    end
    
    if createLocal
      toolTipString = ['Click to create a local copy of the selected '...
                       psFuncName ' sub-module.'];
      psBgColor_tmp = psBgColor_ok2;
      buttonString = 'Create Fcns';
    else
      if psSimStruct.(psFuncName).varInfo.initialized && (strcmp(psControlStruct.(psFuncName).dirNames{psControlStruct.(psFuncName).activeElement},psSimStruct.(psFuncName).varInfo.origin) || strcmp(psControlStruct.(psFuncName).dirNames{psControlStruct.(psFuncName).activeElement},'Local') && ~isempty(strfind(psSimStruct.(psFuncName).varInfo.origin,'Local')))
        toolTipString = ['Click to re-initialize the selected '...
                         psFuncName ' sub-module.'];
        buttonString = 'Re-Initialize';
        psBgColor_tmp = psBgColor_ok;
      else
        toolTipString = ['Click to initialize the selected ' ...
                         psFuncName ' sub-module.'];
        buttonString = 'Initialize';
        psBgColor_tmp = psBgColor_error;
      end    
    end
    
    if strcmp(initEnabledStr,'on')
        initButton = uicontrol(tmpFrameHdl,'style','pushbutton', ...
                               'String',buttonString,'Callback', ...
                         ['psStartUp(''psInitClBck'',''' psFuncName ''');'],...
                               'Position',initButtonPos, ...
                               'BackgroundColor',psBgColor_tmp, ...
                               'Enable','on', ...
                               'SelectionHighlight','off', ...
                               'TooltipString',toolTipString);
    else
        toolTipString = notEnabledReason;
        initButton = uicontrol(tmpFrameHdl,'style','pushbutton', ...
                               'String',buttonString,'Callback', ...
                      ['psStartUp(''displayMessage'', {''Not enabled!'','''',''Reason: ' notEnabledReason '''});'],...
                               'Position',initButtonPos, ...
                               'BackgroundColor',psBgColor_error_2, ...
                               'Enable','on', ...
                               'SelectionHighlight','off', ...
                               'TooltipString',toolTipString);
        
    end
  end

  % Draw the command part
  tmpFramePos = [submoduleXSize+8 ...
                 textBoxYStart-submoduleYDiff*(length(psFuncNames)+1) ...
                 commandXSize ...
                 submoduleYDiff*(length(psFuncNames)+1)] ;

  matlabVersion = version;
  if eval(matlabVersion(1))>6
    tmpFrameHdl = ...
        uipanel(mainFigHandle,'Title','Execute commands:','Unit',...
                'pixels','Position', ...
                tmpFramePos,'SelectionHighlight',...
                'off','BackgroundColor', ...
                psBgColor_light);

    tmpFramePos = 0;
  else
    tmpFrameHdl = mainFigHandle;
  end

  
  % Draw the Control Buttons
  commandButtons = {'open','simulate','plot','cleanup'};
  for ll = 1:length(commandButtons)
      switch commandButtons{ll}
        case 'open'
          buttonString  = 'Open Model';
          psBgColor_tmp = psBgColor_ok;
          buttonVarName = 'openModelButton';
          tmpClBck = ['try, psModel, catch, ' ...
                      'psStartUp(''displayMessage'', {''Something went ' ...
                      'wrong'','', check command window!''}),'...
                      'rethrow(lasterror); ,end'];
          tmpEnabledStr = 'on';
          toolTipString = 'Open Selected Simulation Model';
        case 'simulate'
          buttonString  = 'Simulate';
          psBgColor_tmp = psBgColor_ok;
          buttonVarName = 'simulateButton';
          tmpClBck = ['try, mkSim,psStartUp(''updateMainWindow''), catch, ' ...
                      'psStartUp(''displayMessage'', {''Something went ' ...
                      'wrong'','', check command window!''}),'...
                      'rethrow(lasterror); ,end'];
          tmpEnabledStr = 'on';
          toolTipString = 'Simulation Model';
        case 'plot'
          global psSimOut
          if isempty(psSimOut)
              tmpEnabledStr = 'off';
              notEnabledReason = 'There is no data to plot!';
              psBgColor_tmp = psBgColor_error_2;
          else
              tmpEnabledStr = 'on';
              psBgColor_tmp = psBgColor_ok;
          end
          buttonString  = 'Plot Data';

          buttonVarName = 'plotDataButton';
          tmpClBck = ['try, mkPlots, catch, ' ...
                      'psStartUp(''displayMessage'', {''Something went ' ...
                      'wrong'','', check command window!''}),'...
                      'rethrow(lasterror); ,end'];
          toolTipString = 'Make example plot of simulation data';
        case 'cleanup'
          buttonString  = 'Clean Up';
          psBgColor_tmp = psBgColor_ok;
          buttonVarName = 'cleanupDataButton';
          tmpClBck = ['try, global psSimOut,psSimOut=[],psStartUp(''updateMainWindow''),clear psThermProp, catch, ' ...
                      'psStartUp(''displayMessage'', {''Something went ' ...
                      'wrong'','', check command window!''}),'...
                      'rethrow(lasterror); ,end'];
          tmpEnabledStr = 'on';
          toolTipString = 'Clean up some crashable variables.';
        otherwise
          disp('psStartUp: Error in createMainWindow, this should not happen')
      end

      tmpButtonPos = [psCommandButtonXStart  ...
                      length(psFuncNames)*submoduleYDiff-(commandButtonYDiff*(ll-0.5)-1) ...
                      psCommandButtonXSize ...
                      psCommandButtonYSize];   
      
      tmpButtonPos = tmpButtonPos  + tmpFramePos.*[1 1 0 0];
      if strcmp(tmpEnabledStr,'on')
          tmpButton = uicontrol(tmpFrameHdl,'style','pushbutton', ...
                                'String',buttonString,'Callback', ...
                                tmpClBck,...
                                'Position',tmpButtonPos, ...
                                'BackgroundColor',psBgColor_tmp, ...
                                'Enable','on', ...
                                'SelectionHighlight','off', ...
                                'TooltipString',toolTipString);
      else
          toolTipString = notEnabledReason;
          tmpButton = uicontrol(tmpFrameHdl,'style','pushbutton', ...
                                 'String',buttonString,'Callback', ...
                                 ['psStartUp(''displayMessage'', {''Not enabled!'','''',''Reason: ' notEnabledReason '''});'],...
                                 'Position',tmpButtonPos, ...
                                 'BackgroundColor',psBgColor_error_2, ...
                                 'Enable','on', ...
                                 'SelectionHighlight','off', ...
                                 'TooltipString',toolTipString);
      end
      
      %eval([buttonVarName '= tmpButton;']);

  end

  
  % Draw the current dir part
  tmpPanelTextBoxPos = [4 6 ...
                   mainFigXSize-8 (19*3)+20]; 

  matlabVersion = version;
  if eval(matlabVersion(1))>6
    tmpInfoFrameHdl = ...
        uipanel(mainFigHandle,'Title','Information:','Unit',...
                'pixels','Position', ...
                tmpPanelTextBoxPos,'SelectionHighlight',...
                'off','BackgroundColor', ...
                psBgColor_light);
    tmpPanelTextBoxPos = 0;
  else
    tmpInfoFrameHdl = mainFigHandle;
  end

  tmpTextBoxPos = [4 1 140 19]; 
  tmpTextBoxPos = tmpTextBoxPos+tmpPanelTextBoxPos.*[1 1 0 0];
  uiCtrl(1) = uicontrol('Parent',tmpInfoFrameHdl,'style','text','String', ...
                        'Simulation dir:','Position', ...
                        tmpTextBoxPos,'SelectionHighlight','off',...
                        'HorizontalAlignment','Left','BackgroundColor', ...
                        psBgColor_light);
  tmpTextBoxPos = [141 1 400 19]; 
  tmpTextBoxPos = tmpTextBoxPos+tmpPanelTextBoxPos.*[1 1 0 0];
  uiCtrl(2) = uicontrol('Parent',tmpInfoFrameHdl,'style','text','String', ...
                        psControlStruct.currentSimDir,'Position', ...
                        tmpTextBoxPos,'SelectionHighlight','off',...
                        'FontAngle','Italic',...
                        'HorizontalAlignment','Left','BackgroundColor', ...
                        psBgColor_light);

  tmpTextBoxPos = [4 20 140 19];
  tmpTextBoxPos = tmpTextBoxPos+tmpPanelTextBoxPos.*[1 1 0 0];
  uiCtrl(3) = uicontrol('Parent',tmpInfoFrameHdl,'style','text','String', ...
                        'psDist dir: ','Position', ...
                        tmpTextBoxPos,'SelectionHighlight','off',...
                        'HorizontalAlignment','Left',...
                        'BackgroundColor', psBgColor_light);  
  tmpTextBoxPos = [141 20 400 19];
  tmpTextBoxPos = tmpTextBoxPos+tmpPanelTextBoxPos.*[1 1 0 0];
  uiCtrl(4) = uicontrol('Parent',tmpInfoFrameHdl,'style','text','String', ...
                        psControlStruct.distDir,'Position', ...
                        tmpTextBoxPos,'SelectionHighlight','off',...
                        'FontAngle','Italic',...
                        'HorizontalAlignment','Left','BackgroundColor', ...
                        psBgColor_light);


  tmpTextBoxPos = [4 39 140 19];
  tmpTextBoxPos = tmpTextBoxPos+tmpPanelTextBoxPos.*[1 1 0 0];
  uiCtrl(5) = uicontrol('Parent',tmpInfoFrameHdl,'style','text','String', ...
                        'Current dir: ','Position', ...
                        tmpTextBoxPos,'SelectionHighlight','off',...
                        'HorizontalAlignment','Left','BackgroundColor', ...
                        psBgColor_light);  
  tmpTextBoxPos = [141 39 400 19];
  tmpTextBoxPos = tmpTextBoxPos+tmpPanelTextBoxPos.*[1 1 0 0];
  uiCtrl(6) = uicontrol('Parent',tmpInfoFrameHdl,'style','text','String', ...
                        pwd,'Position', ...
                        tmpTextBoxPos,'SelectionHighlight','off',...
                        'FontAngle','Italic',...
                        'HorizontalAlignment','Left','BackgroundColor', ...
                        psBgColor_light);


  
  % Add simple-but cool menu-functionality
  for tmpIdx = 1:2
    switch tmpIdx 
     case 1
      theMenuHandle = mainFigHandle;
     case 2
      tmpMainfigMenu = uicontextmenu ;
      set(mainFigHandle,'UIContextMenu',tmpMainfigMenu);
      set(tmpFrameHdl,'UIContextMenu',tmpMainfigMenu);
      set(tmpInfoFrameHdl,'UIContextMenu',tmpMainfigMenu);
      set(tmpInfoFrameHdl,'UIContextMenu',tmpMainfigMenu);
      set(titleText,'UIContextMenu',tmpMainfigMenu);
      for uiidx = 1:length(uiCtrl)
        set(uiCtrl(uiidx),'UIContextMenu',tmpMainfigMenu);      
      end
      theMenuHandle = tmpMainfigMenu;
    end

    
    fileMenu = uimenu(theMenuHandle,'Label','File');
    uimenu(fileMenu,'Callback','psStartUp(''psNewSimulClbck'');',...
         'Label','New simulation');
    uimenu(fileMenu,'Callback','psStartUp(''psChangeDirClbck'');',...
         'Label','Change simulation dir');
    uimenu(fileMenu,'Callback','psStartUp(''psLoadSimulClbck'');',...
           'Label','Load simulation data');
    uimenu(fileMenu,'Callback','psStartUp(''psOpenSimulClbck'');',...
           'Label','Open simulation in dir');
    uimenu(fileMenu,'Callback','psStartUp(''psSaveSimulClbck'');',...
           'Label','Save Simulation');
    uimenu(fileMenu,'Callback',...
           ['cd(''' psControlStruct.currentSimDir ''');'...
            'disp(''Changing directory'');psStartUp(''updateMainWindow'');'],...
           'Label','Go to simDir');
    uimenu(fileMenu,'Callback',['psStartUp(''psCloseClbck'',' ...
                        num2str(mainFigHandle) ');'],...
           'Label','Close psPack');
    
    helpMenu = uimenu(theMenuHandle,'Label','Help');
    uimenu(helpMenu,'Callback','psStartUp(''psAboutClbck'');',...
           'Label','About');
    uimenu(helpMenu,'Callback','psStartUp(''psDocClbck'');',...
           'Label','Documentation');
  end    
  % -------------------------------------------------------------------------
  % End Method: createMainWindow
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : psListBoxClBck 
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- psFuncName
  % -------------------------------------------------------------------------
  % Description : Executes the operations needed when a listbox has
  %               changed value. Normally this function is called
  %               as a callback from a listbox which passes the
  %               actual psFuncName in arg1.
  % -------------------------------------------------------------------------
 case 'psListBoxClBck'
  psFuncName = arg1;
  try 
    newActiveElement = get(psControlStruct.(psFuncName).listBox, 'Value');
    if psControlStruct.(psFuncName).activeElement(1) ~= ...
          newActiveElement
      if ~strcmp(psControlStruct.(psFuncName).dirNames{psControlStruct.(psFuncName).activeElement},'Local')
        psControlStruct.(psFuncName).activeElement = newActiveElement;
      end
      psStartUp('updateControlStruct',psFuncName);
      psStartUp('updateMainWindow');    
    end
  catch
    h = warndlg(['A callback-routine failed to execute. A probable ' ...
                 'reason is that you tried to execute it from the ' ...
                 'command line.'],'!! Warning !!');  
    set(h,'WindowStyle','modal');
    uiwait(h);
    outValue = -1;
  end
  % -------------------------------------------------------------------------
  % End Method: psListBoxClBck
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : psInfoClBck
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- psFuncName
  % -------------------------------------------------------------------------
  % Description : Executes operations necessary after a info-button
  %               has been pressed. Usually this function is called
  %               as a callback from an information button with the
  %               actual psFuncName in arg1.
  % -------------------------------------------------------------------------
 case 'psInfoClBck'
  psFuncName = arg1;
  readmeFileName = [psFuncName 'Readme.txt'];
  activeElement = psControlStruct.(psFuncName).activeElement;
  activeDirectory = psControlStruct.(psFuncName).dirPaths{activeElement};
  
  readmeInfoFile = dir([activeDirectory filesep readmeFileName]);

  if ~isempty(readmeInfoFile)
    fid = fopen([activeDirectory filesep readmeFileName],'r');
    F = fread(fid);
    fclose(fid);
    message = char(F');
    %    sizeDiff = ceil(length(message)/20)*20-length(message);
    %    if  sizeDiff
    %      message = [message repmat(' ',1,sizeDiff)];
    %    end
    %    message = reshape(message,20,[])';
    psStartUp('displayMessage',message);
  else
    h = warndlg(['File ' readmeFileName ' not found!'],'Warning');
  end
  % -------------------------------------------------------------------------
  % End Method: psInfoClBck
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : psInitClBck 
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- psFuncName
  % -------------------------------------------------------------------------
  % Description : Executes operations necessary after a init/create-fcns
  %               button has been pressed. Usually this function is called
  %               as a callback from an init button with the
  %               actual psFuncName in arg1.
  % -------------------------------------------------------------------------
 case 'psInitClBck'
  psFuncName = arg1;
  workingDir = psControlStruct.currentSimDir;
  executable = [psFuncName 'Init(''init'',''' workingDir ''',1,' ...
                ' psSimStruct.(psFuncName))'];
  figNbr = eval(executable);
  if figNbr 
    set(psControlStruct.mainFigure.handle,'Visible','off');
    uiwait(figNbr);
    set(psControlStruct.mainFigure.handle,'Visible','on');
  end
  if isfield(psSimStruct.(psFuncName).varInfo,'localCreated') && ...
        psSimStruct.(psFuncName).varInfo.localCreated
    psSimStruct.(psFuncName).varInfo=...
        rmfield(psSimStruct.(psFuncName).varInfo,'localCreated');
    if psControlStruct.(psFuncName).localExists
      activeElement = length(psControlStruct.(psFuncName).dirNames);
    else
      activeElement = length(psControlStruct.(psFuncName).dirNames)+1;
    end
    psControlStruct.(psFuncName).activeElement = activeElement;
  end
  
%  if isempty(psSimStruct.(psFuncName))
%    psSimStruct.(psFuncName).varInfo.initialized = 0;
%  end
  
  
  psStartUp('updateControlStruct');
  psStartUp('updateMainWindow');
  % -------------------------------------------------------------------------
  % End Method: psInitClBck
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : psClearClBck 
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- psFuncName
  % -------------------------------------------------------------------------
  % Description : Executes operations necessary after a clear button
  %               has been pressed. Usually this function is called
  %               as a callback from a clear button with the
  %               actual psFuncName in arg1.
  % -------------------------------------------------------------------------
 case 'psClearClBck'
  psFuncName = arg1;
  tmpNames = psControlStruct.(psFuncName).([psFuncName 'Files']); 
  tmpLngth = length(tmpNames);

  nameListMsg = [];
  for tmpIdx = 1:tmpLngth
    nameListMsg = ...
        [nameListMsg psControlStruct.(psFuncName).dirPaths{end} ...
         filesep tmpNames{tmpIdx} '.old'];
    if tmpIdx < tmpLngth-1
      nameListMsg = [nameListMsg ', '];
    elseif tmpIdx < tmpLngth
      nameListMsg = [nameListMsg ' and '];
    else    
      nameListMsg = [nameListMsg '.'];
    end
  end
  
  if psControlStruct.(psFuncName).localExists
    warnmsg = ['Are you sure? This will, if they exists, replace ' ...
               'the files: ' ...
               nameListMsg];      
    button = questdlg(warnmsg, 'psPack (Warning)','Yes','No','No');
    if strcmp(button,'Yes')
      for tmpIdx = 1:tmpLngth
        eval(['!mv ' psControlStruct.(psFuncName).dirPaths{end} filesep ...
              tmpNames{tmpIdx} ' '...
              psControlStruct.(psFuncName).dirPaths{end} filesep...
              tmpNames{tmpIdx} '.old']);
      end
      rehash path; % Rehash paths to make sure matlab knows
                   % that (psFuncName)Init dissappeared
      psSimStruct.(psFuncName) = [];
      psSimStruct.(psFuncName).varInfo.initialized = 0;
      psStartUp('updateControlStruct');
      psStartUp('updateMainWindow');
    end
  end
  % -------------------------------------------------------------------------
  % End Method: psClearClBck
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : psAboutClbck 
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Displays the about-popup. Usually this function
  %               is called as a callback from the menu, but it is
  %               also possible to call this function from the
  %               command line.
  % -------------------------------------------------------------------------
 case 'psAboutClbck'
  aboutFig = figure;
  clf;

  scrollMessage = {'psPack - a Process Simulation Package';...
                   '';...
                 'This product comes without any warranty. You may freely';...
                   'use this product for any purpose.';...
                   '';...
                   'This product is licensed under the GPL license.';...
                   '';...
                   '-----------------------------------------------';...
                   '';...
                   'This program package is founded on the common ';...
                   'work of the people working at Vehicular Systems at ';...
                   'Link�ping University in Sweden. Without these people ';...
                   'psPack would never have been put together.';...
                   '';...
                   'Thanks for your contributions:';...
                   '';...
                   'Lars Eriksson';...
                   '';...
                   'Per Andersson';...
                   'Gunnar Cedersund';...
                   'Anders Fr�berg';...
                   'Marcus Klein';...
                   'Ylva Nilsson';...
                   '';...
                   '-----------------------------------------------';...
                   '';...
                   '';...
                   '';...
                   '';...
                   '';...
                   ''};
  aboutFigPosition = get(psControlStruct.mainFigure.handle,'Position') ...
      + [30 30 0 0]; 

  set(aboutFig,'MenuBar','none','Color',psBgColor_light, ...
               'Position',[ aboutFigPosition(1) ...
                      aboutFigPosition(2) 348 300],...
               'Name','psPack - 1.0 (About)',...
               'NumberTitle','off','Resize','off',...
               'WindowStyle','modal')

  titleText = uicontrol(aboutFig,'style','text', ...
                'String','-^- psPack -^-','FontSize',32,...                    
                        'BackgroundColor',psBgColor_light, ...
                        'Position',[0 235 348 50], ...
                        'SelectionHighlight','off',...
                        'TooltipString','psPack - 1.0 (About)');

  underTitleText = uicontrol(aboutFig,'style','text', ...
                             'String',{'The Process Simulation Package';...
                      'Copyright (c) Per �berg'},...
                             'FontSize',12,...
                             'Position',[20 180 319 52], ...
                             'BackgroundColor',psBgColor_light, ...
                             'SelectionHighlight','off',...
                             'TooltipString','psPack - 1.0 (About)',...
                             'HorizontalAlignment','Left');
  uicontrol(aboutFig,'style','text','String',...
            'Information:',...
            'FontSize',8,...
            'Position',[24 160 77 15], ...
            'BackgroundColor',psBgColor_light, ...
            'SelectionHighlight','off',...
            'TooltipString','psPack - 1.0 (About)',...
            'HorizontalAlignment','Center');

  frameHandle = uicontrol(aboutFig,'style','frame',...
                          'Position',[24 18 304 146], ...
                          'BackgroundColor',psBgColor_light, ...
                          'SelectionHighlight','off',...
                          'TooltipString','psPack - 1.0 (About)');

  aboutText = uicontrol(aboutFig,'style','text', ...
                        'String',scrollMessage,...
                        'FontSize',8,...
                        'Position',[34 29 283 120], ...
                        'BackgroundColor',psBgColor_light, ...
                        'SelectionHighlight','off',...
                        'TooltipString','psPack - 1.0 (About)',...
                        'HorizontalAlignment','Left');
  
  toolTipString = ['Click OK or close window to return.'];
  okButton = uicontrol(aboutFig,'style','pushbutton', ...
                       'String','pUsH-mE','Callback', ...
                       'closereq', ...
                       'Position',[262 11 67 23], ...
                       'BackgroundColor',psBgColor_light*0.9, ...
                    'SelectionHighlight','off','TooltipString',toolTipString); 

  textLength = 9;
  scrollMsgLength = length(scrollMessage);
  tmpIdx = 1;

  for i = 1:textLength
    scrollMessage{i+scrollMsgLength} = scrollMessage{i};
  end

  pause(5)
  while ishandle(aboutFig)
    tmpMessage = scrollMessage(tmpIdx:(tmpIdx+textLength)) ;
    set(aboutText,'String',tmpMessage);
    tmpIdx = tmpIdx +1;
    pause(1)
    if tmpIdx > scrollMsgLength
      tmpIdx = 1;
    end
  end
  % -------------------------------------------------------------------------
  % End Method: psAboutClbck
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : psNewSimulClbck 
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Resests psControlStruct and psSimStruct and asks
  %               questions about new directories. This function is
  %               usually called as a callback from the menu. Even
  %               though there is no obvious reason why this function
  %               should ever be called from the commandline it should
  %               be possible.
  % -------------------------------------------------------------------------
 case 'psNewSimulClbck'
  set(psControlStruct.mainFigure.handle,'Visible','off')
  newCurrentDir =  uigetdir(psControlStruct.currentSimDir,...
                            'Select a directory for your new Simulation');
  if  newCurrentDir
    % Prepare for initControlStruct
    cd(newCurrentDir);
    mainFig = psControlStruct.mainFigure.handle;
    rmpath(psControlStruct.currentSimDir);
    
    psStartUp('initControlStruct');
    
    % Fix back things that initControlStruct messed up
    psControlStruct.mainFigure.handle = mainFig;
    
    % Now initialize simStruct
    psStartUp('initSimStruct');
    
    % And finally do the normal things
    psStartUp('updateControlStruct');
    psStartUp('updateMainWindow'); 
  else
    set(psControlStruct.mainFigure.handle,'Visible','on')
    psStartUp('updateControlStruct');
    psStartUp('updateMainWindow');
  end
  % -------------------------------------------------------------------------
  % End Method: psNewSimulClbck
  % -------------------------------------------------------------------------
 
  % -------------------------------------------------------------------------
  % Method      : psChangeDirClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Sets psControlStruct.currentSimDir and changes directory.
  % -------------------------------------------------------------------------
 case 'psChangeDirClbck'
  newCurrentDir =  uigetdir(psControlStruct.currentSimDir,...
                            ['Select a directory for your new ' ...
                      'Simulation']);
  if newCurrentDir
    rmpath(psControlStruct.currentSimDir);
    psControlStruct.currentSimDir = newCurrentDir;
    cd(newCurrentDir);
    addpath(newCurrentDir);
    psStartUp('updateControlStruct');
    psStartUp('updateMainWindow');
  end
  % -------------------------------------------------------------------------
  % End Method: psChangeDirClbck
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : psCloseClbck
  % -------------------------------------------------------------------------
  % Arguments   : mainFigure <- arg1 , softShutDown <- arg2
  % -------------------------------------------------------------------------
  % Description : Closes psPack and clears local/persistent and
  %               global variables.
  % -------------------------------------------------------------------------
 case 'psCloseClbck'

  switch nargin
   case 2
    mainFigure = arg1;
    softShutDown = 0;
   case 3
    mainFigure = arg1;
    softShutDown = arg2;
   otherwise
    outValue =  -1;
    return
  end

  if ~softShutDown 
    button = questdlg('This will end your psPack session. Are you sure?', ...
                      'psPack 1.0 (Warning)','Yes','No','No');
    if strcmp(button,'Yes')                      
      if  psControlStruct.currentSimDir
        rmpath(psControlStruct.currentSimDir);
      end
      if mainFigure && ishandle(mainFigure)
        delete(mainFigure);
      end
    clear global psSimStruct;
    clear psControlStruct;
    munlock
    end    
  else 
    if  ~isempty(psControlStruct) && ~isempty(psControlStruct.currentSimDir)
      rmpath(psControlStruct.currentSimDir);
    end
    if mainFigure && ishandle(mainFigure)
      delete(mainFigure);
    end
    psSimStruct = [];
    psControlStruct = [];
  end
  % -------------------------------------------------------------------------
  % End Method: psCloseClbck
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : psDocClbck
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- psDocPage
  % -------------------------------------------------------------------------
  % Description : This function starts a web-browser that points to
  %               the documentation. If no argument is passed, the
  %               main documentation page is displayed. If an argument
  %               is passed. The page requested in arg1 is displayed.
  % -------------------------------------------------------------------------
 case 'psDocClbck'
  if nargin >= 2
    psDocPage = arg1;
  else
    psDocPage = 'index.html';
  end
  
  psDocUrl = [psControlStruct.distDir filesep '..' filesep 'psDoc' ...
              filesep psDocPage]; 
  web(['file:///' psDocUrl], '-browser')
  % -------------------------------------------------------------------------
  % End Method: psDocClbck
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : psLoadSimulClbck 
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Displays a dialog that asks about a new directory to
  %               start a new simulation in. Usually this function is
  %               called as a callback from the menu. If an error
  %               occurs nothing special happens. The difference
  %               between load and open-simul is that load does not
  %               change directory.
  % -------------------------------------------------------------------------
 case 'psLoadSimulClbck'
  [fileName, pathName] = uigetfile('*.mat', 'Select a saved simulation.');
  if fileName
    try 
      loadStruct = load([pathName filesep fileName]);
    catch
      loadStruct = [];
    end
    if isfield(loadStruct,'psSimStruct')
      notAnswered = 0;            
      % Prepare for initControlStruct
      mainFig = psControlStruct.mainFigure.handle;

      psStartUp('initControlStruct');
      
      % Fix back things that initControlStruct messed up
      psControlStruct.mainFigure.handle = mainFig;
      
      % Now initialize simStruct
      psStartUp('initSimStruct');
    
      % Get on with the normal operation
      psStartUp('updateControlStruct');
      psStartUp('loadSimStruct',loadStruct.psSimStruct);
      psStartUp('updateControlStruct'); % To correctly set new paths
      psStartUp('updateMainWindow');
      outValue = 0;
    else
      h = warndlg('No previous simulation-setup found in file.',...
                  '!! Warning !!');
      set(h,'WindowStyle','modal');
      uiwait(h);
      outValue = -1;
    end
  else
    outValue = -1;      
  end
  % -------------------------------------------------------------------------
  % End Method: psLoadSimulClbck
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : psOpenSimulClbck 
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Displays a dialog that asks about a new directory to
  %               start a new simulation in. Usually this function is
  %               called as a callback from the menu. If an error
  %               occurs nothing special happens.The difference
  %               between load and open-simul is that open changes directory
  % -------------------------------------------------------------------------
 case 'psOpenSimulClbck'
  [fileName, pathName] = uigetfile('*.mat', 'Select a saved simulation.');
  if fileName
    try 
      loadStruct = load([pathName filesep fileName]);
    catch
      loadStruct = [];
    end
    if isfield(loadStruct,'psSimStruct')
      notAnswered = 0;            
      % Prepare for initControlStruct
      cd(pathName)
      mainFig = psControlStruct.mainFigure.handle;
      rmpath(psControlStruct.currentSimDir);
    
      psStartUp('initControlStruct');
      
      % Fix back things that initControlStruct messed up
      psControlStruct.mainFigure.handle = mainFig;
      
      % Now initialize simStruct
      psStartUp('initSimStruct');
    
      % Get on with the normal operation
      psStartUp('updateControlStruct');
      psStartUp('loadSimStruct',loadStruct.psSimStruct);
      psStartUp('updateControlStruct'); % To correctly set new paths
      psStartUp('updateMainWindow');
      outValue = 0;
    else
      h = warndlg('No previous simulation-setup found in file.',...
                  '!! Warning !!');
      set(h,'WindowStyle','modal');
      uiwait(h);
      outValue = -1;
    end
  else
    outValue = -1;      
  end
  % -------------------------------------------------------------------------
  % End Method: psOpenSimulClbck
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : psSaveSimulClbck 
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Displays a dialog that asks about a new directory to
  %               start a new simulation in. Usually this function is
  %               called as a callback from the menu. If an error
  %               occurs nothing special happens.
  % -------------------------------------------------------------------------
 case 'psSaveSimulClbck'
  [fileName, pathName] = uiputfile('.mat', 'Choose a filename...');
  if fileName
    try 
      save([pathName filesep fileName],'psSimStruct','-V6');
    catch
      h = warndlg(['Could not write to file: ' fileName]);
      set(h,'WindowStyle','modal');
      uiwait(h);
    end
  else
    outValue = -1;      
  end

  
  % -------------------------------------------------------------------------
  % End Method: psSaveSimulClbck
  % -------------------------------------------------------------------------
  
  
  % -------------------------------------------------------------------------
  % Method      : displayMessage
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- message
  % -------------------------------------------------------------------------
  % Description : Displays the message passed in arg1. The window
  %               is then made modal so that the user has to press
  %               a button to return to the calling program.
  % -------------------------------------------------------------------------
 case 'displayMessage'
  message = arg1;

  okButtonFig = figure;
  clf;
  messagePosition = get(psControlStruct.mainFigure.handle,'Position') ...
      + [30 30 0 0]; 
  set(okButtonFig,'MenuBar','none','Color',psBgColor_light, ...
                  'Position',[ messagePosition(1) ...
                      messagePosition(2) 320 200],...
                  'Name','psPack (Message)','NumberTitle','off',...
                  'Resize','off')
  
  okButtonPos = [250 10 40 20 ];  
  toolTipString = ['Click OK or close window to return.'];
  okButton = uicontrol(okButtonFig,'style','pushbutton', ...
                       'String','OK','Callback', ...
                       'closereq', ...
                       'Position',okButtonPos, ...
                       'BackgroundColor',psBgColor_dark, ...
                       'SelectionHighlight','off','TooltipString',...
                       toolTipString); 
  okMessagePos = [25 180-150 320-50 150 ];  
  okMessage = uicontrol(okButtonFig,'style','text', ...
                        'String',message,...
                        'Position',okMessagePos, ...
                        'BackgroundColor',psBgColor_light, ...
                        'HorizontalAlignment','left', ...
                        'SelectionHighlight','off','TooltipString',...
                        toolTipString);  
  set(okButtonFig,'WindowStyle','modal')
  % -------------------------------------------------------------------------
  % End Method: displayMessage
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : getControlStruct
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Returns the current psControlStruct. This
  %               function is mainly for debugging purposes and is
  %               in no way related to any inter-function-communication.
  % -------------------------------------------------------------------------
 case 'getControlStruct'
  outValue = psControlStruct;
  % -------------------------------------------------------------------------
  % End Method: getControlStruct
  % -------------------------------------------------------------------------
 otherwise
  disp('Allowed methods')    
  disp(['      startUp                : Initializes the program-' ...
        'controlstructs']) 
  disp('                               from scratch. ')
  disp('      initControlStruct      : Set up the most basic controlstruct')
  disp('                               data. Then automatically run')
  disp('                               ''updateControlStruct''')
  disp(['      initSimStruct          : Basic initialization of ' ...
        'the psSimStruct.']) 
  disp(['      updateControlStruct    : Reads controlinformation ' ...
        'that is needed to ']) 
  disp('                               correctly draw window elements')
  disp('                               etc. This function checks the local')
  disp(['                               directory for psPack-stuff ' ...
        'and sets the']) 
  disp('                               paths accordingly.')
  disp(['      loadSimStruct          : Sets the psSimStruct ' ...
        'variable to some']) 
  disp('                               reasonable values-according to the')
  disp('                               psSimStruct in arg1, but with care')
  disp(['                               taken to the actual ' ...
        'psPack-dist-directory.']) 
  disp(['      createMainWindow       : Create the mainWindow. ' ...
        'Destroy the old']) 
  disp('                               window and create a new one if there')
  disp('                               happens to be one already.')
  disp(['      updateMainWindow       : Sets mainWindow-elements ' ...
        'to actual values']) 
  disp('                               from controlStruct. In the meantime')
  disp(['                               this function just calls ' ...
        '''createMainWindow''']) 
  disp('      psListBoxClBck         : Executes operations necesarry after')
  disp('                               chage of listbox values.')
  disp('      psInfoClBck            : Executes operations necessary when a')
  disp('                               information-button ha been pressed.')
  disp(['      psInitClBck            : Executes necessary operations ' ...
        'after a']) 
  disp(['                               crete-fcn or init-button ' ...
        'has been pressed']) 
  disp('      psClearClBck           : Clears away local copy of actual')
  disp('                               psFuncName- functionality.')
  disp('      psAboutClbck           : Displays about-popup-information')
  disp('      psChangeDirClbck       : Sets psControlStruct.currentSimDir and')
  disp('                               changes directory. ')
  disp('      psNewSimulClbck        : Executes operations necessary when a')
  disp('                               new simulation is wanted.  ')
  disp('      psCloseClbck           : Shuts psPack down and clears')
  disp(['                               local/persistent and global ' ...
        'variables  ']) 
  disp('      psDocClbck             : Ececutes documentation-operations. ')
  disp('                               Without in-arguments a browser is')
  disp(['                               pointed to the main ' ...
        'documentation-page.']) 
  disp('      psLoadSimulClbck       : Popups a ''new-simulation''')
  disp(['                               question-dialog. If ' ...
        '''stealth'' mode is']) 
  disp('                               wanted loadSimStruct can be used')
  disp('                               instead ')
  disp('      psOpenSimulClbck       : Popups a ''new-simulation''')
  disp(['                               question-dialog. If ' ...
        '''stealth'' mode is']) 
  disp('                               wanted loadSimStruct can be used')
  disp(['                               instead. OpenSimul also ' ...
        'changes directory '])
  disp(['      psSaveSimulClbck       : Saves the current simulation ' ...
        'to file.']) 
  disp(['      displayMessage         : Function to display a ' ...
        'message. This could']) 
  disp('                               be any message, but this function is')
  disp('                               usually called by psInfoClbck ')
  disp('      getControlStruct       : Returns the current psControlStruct,')
  disp(['                               mainly needed for debuging ' ...
        'purposes     ']) 
end

% ===========================================================================
%  Finish  up
% ===========================================================================

if ~strcmp(method , 'psCloseClbck')
  psSimStructCopy = psSimStruct;
end
