%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

% psStripToLocalData
%
% Usage: 
% psSimStruct.psXXXX = psStripToLocalData(psSimStruct.psXXXX)
%
% Description:
% This is an internal function that takes a psSimStruct.psXXXX and
% make a local version of it. This function strips away everything
% that is not optimizable of the psSimStruct.psXXXX and replaces
% its depVarFuncs with new functions where non-optimizable parts
% have been replaced by the corresponding value-strings


function  psFuncParLocal = psStripToLocalData(psFuncPar)

% Basic initialization, so that all variables go in the same
% order as we are used to
psFuncParLocal = [];
psFuncParLocal.varInfo = [];
psFuncParLocal.varInfo.varNames = [];
psFuncParLocal.varInfo.varTypes = [];
psFuncParLocal.varInfo.units = [];
psFuncParLocal.varInfo.unitConversionFactors = [];
psFuncParLocal.varInfo.displayedUnitIdx = [];
psFuncParLocal.varInfo.varDesc = [];
psFuncParLocal.varInfo.depVarNames = [];
psFuncParLocal.varInfo.depVarFuncs = [];
psFuncParLocal.varInfo.freeVarNames = [];
psFuncParLocal.varInfo.optimVarNames = [];
psFuncParLocal.varInfo.optimCtrl = [];
psFuncParLocal.varInfo.psPackVersion = psFuncPar.varInfo.psPackVersion;
psFuncParLocal.varInfo.origin = [psFuncPar.varInfo.origin 'Local'];

% Firstly wee need to add 
% 'optimVarNames' to the new variable list. ... 
for optimIdx = 1:length(psFuncPar.varInfo.optimVarNames)
  varName = psFuncPar.varInfo.optimVarNames{optimIdx};        
  ii = find(strcmp(psFuncPar.varInfo.varNames,varName));
  psFuncParLocal.(varName) = psFuncPar.(varName);
  psFuncParLocal.varInfo.varNames{optimIdx} = ...
      psFuncPar.varInfo.varNames{ii};
  psFuncParLocal.varInfo.varTypes{optimIdx} = ...
      psFuncPar.varInfo.varTypes{ii};
  psFuncParLocal.varInfo.units{optimIdx} = ...
      psFuncPar.varInfo.units{ii};
  psFuncParLocal.varInfo.unitConversionFactors{optimIdx} = ...
      psFuncPar.varInfo.unitConversionFactors{ii};
  psFuncParLocal.varInfo.displayedUnitIdx{optimIdx} = ...
      psFuncPar.varInfo.displayedUnitIdx{ii};
  psFuncParLocal.varInfo.varDesc{optimIdx} = ...
        psFuncPar.varInfo.varDesc{ii};
  psFuncParLocal.varInfo.optimVarNames{optimIdx} = ...
      psFuncPar.varInfo.optimVarNames{optimIdx};
  psFuncParLocal.varInfo.optimCtrl{optimIdx} = ...
      psFuncPar.varInfo.optimCtrl{optimIdx};
end
  
% ... Then we need to add those of 'depVarNames'
% that depends on already added variables in 'varNames'
% and their 'depVarFuncs' where variables that are not
% already added to 'varNames' is replaces with their value. 

% Prepare for adding new variables by setting some idxs's 
varIdx = length(psFuncParLocal.varInfo.varNames) + 1;      
newDepVarIdx = 1;
for depVarIdx  = 1:length(psFuncPar.varInfo.depVarNames)
  varName = psFuncPar.varInfo.depVarNames{depVarIdx};
  oldVarIdx = find(strcmp(psFuncPar.varInfo.varNames,varName));
  
  % Now we need to start by checking if the variable depends
  % on some of the already added variables for this we need
  % some preparation
  
  % 1) Get the function out
  depVarIdx = find(strcmp(psFuncPar.varInfo.depVarNames,varName));
  theFun = psFuncPar.varInfo.depVarFuncs{depVarIdx};

  % 2) Get its dependecies of the function
  theDepVars = symvar(theFun);
  % 3) Now, does it depend on some of the already added
  %    varNames. Since the depVars are added in their order
  %    of calculation we cannot miss any dependecies
  tmpTestVal = [];
  for kk = 1:length(theDepVars)
    tmpTestVal(kk) = ~isempty(find(strcmp(theDepVars{kk}, ...
                                          psFuncParLocal.varInfo ...
                                          .varNames)));  
  end

  % If it should be added, then....
  if max(tmpTestVal)
    % Add the easy part
    psFuncParLocal.varInfo.varNames{varIdx} = ...
        psFuncPar.varInfo.varNames{oldVarIdx};
    psFuncParLocal.(varName) = psFuncPar.(varName);
    psFuncParLocal.varInfo.varTypes{varIdx} = ...
        psFuncPar.varInfo.varTypes{oldVarIdx};
    psFuncParLocal.varInfo.units{varIdx} = ...
        psFuncPar.varInfo.units{oldVarIdx};
    psFuncParLocal.varInfo.unitConversionFactors{varIdx} = ...
        psFuncPar.varInfo.unitConversionFactors{oldVarIdx};
    psFuncParLocal.varInfo.displayedUnitIdx{varIdx} = ...
        psFuncPar.varInfo.displayedUnitIdx{oldVarIdx};
    psFuncParLocal.varInfo.varDesc{varIdx} = ...
        psFuncPar.varInfo.varDesc{oldVarIdx};
    psFuncParLocal.varInfo.depVarNames{newDepVarIdx} = ...
        psFuncPar.varInfo.depVarNames{depVarIdx};
    
    % ---------------------------------
    % Now create new inline function (not so easy part)
    % ---------------------------------

    % Start by parsing the old inline function and finding
    % out which parts could be variable names
    %  From the Matlab documentation:
    %  -  A valid variable name is a character string of letters,
    %  -  digits, and underscores, totaling not more than
    %  -  namelengthmax characters and beginning with a letter 
    tmpFun = psFuncPar.varInfo.depVarFuncs{depVarIdx};
    tmpOldStr = char(tmpFun);
    
    idxArray = [];
    idxArrayIdx = 1;
    beginNew = 1;
    for iii = 1:length(tmpOldStr)
      if beginNew 
        if isletter(tmpOldStr(iii))
          idxArray{idxArrayIdx} = [iii];
          beginNew = 0;
        end        
      else
        if isvarname([tmpOldStr(idxArray{idxArrayIdx}) tmpOldStr(iii)])
          idxArray{idxArrayIdx} = [idxArray{idxArrayIdx} iii];
        else
          beginNew=1;
          idxArrayIdx = idxArrayIdx+1;
        end
      end      
    end
    
    % Now we have the idxArray who tells us what possible
    % varnames we have in the expression. Lets check which of
    % idxArray is varNames but not already localVarNames 
    jj = 1;
    newIdxArray = [];
    for kk = 1:length(idxArray)
      tmpName = tmpOldStr(idxArray{kk});
      if ~isempty(find(strcmp(psFuncPar.varInfo.varNames,...
                              tmpName))) && ...
            isempty(find(strcmp(psFuncParLocal.varInfo.varNames,...
                                tmpName)))
        newIdxArray{jj} = idxArray{kk};
        jj = jj+1;
      end              
    end
    
    % newIdxArray now contains the indexses to the parts of
    % the strings that should be replaced. (I.e. pars of idxArray)
    
    % Now we chop up the tmpOldStr whihc contains the old
    % expression in textual form and make a list of all parts
    % For every part we set a flag in the vector 'isReplacePart'
    finalIdxArray = [];
    isReplacePart = [];
    partIdx = 1;
    if ~isempty(newIdxArray)
      if min(newIdxArray{1}) ~= 1
        finalIdxArray{partIdx} = 1:(min(newIdxArray{1})-1);
        isReplacePart{partIdx} = 0;
        partIdx = partIdx+1;
      end
      for kk = 1:length(newIdxArray)
        finalIdxArray{partIdx} = newIdxArray{kk};
        isReplacePart{partIdx} = 1;
        partIdx = partIdx + 1;
        if kk==length(newIdxArray)
          if max(newIdxArray{kk}) ~= length(tmpOldStr)
            finalIdxArray{partIdx} = (max(newIdxArray{kk})+1): ...
                length(tmpOldStr); 
            isReplacePart{partIdx} = 0;
            partIdx = partIdx+1;
          end
        else
          finalIdxArray{partIdx} = (newIdxArray{kk}+1): ...
              (newIdxArray{kk+1}-1);                 
          isReplacePart{partIdx} = 0;
          partIdx = partIdx+1;                
        end
      end

      % Now isReplacePart and finalIdxArray is finished. We
      % now need to parse the finalIdxArray and replace
      % parts that should be replaced.
      finalExpr = [];
      for kk = 1:length(finalIdxArray)
        if isReplacePart{kk}
          % get the varName  here !!!
          theName = tmpOldStr(finalIdxArray{kk});
          finalExpr = [finalExpr ...
                       num2str(psFuncPar.(theName))];
        else
          finalExpr = [finalExpr tmpOldStr(finalIdxArray{kk})] ;
        end
      end
    else
      finalExpr = tmpOldStr;
    end
    % Make the expression an inline function
    psFuncParLocal.varInfo.depVarFuncs{newDepVarIdx} = inline(finalExpr);
    
    % Make sure that the final expression does depend on
    % some variable. I dont think this will ever happen.
    if isempty(symvar(finalExpr))
      psFuncParLocal.varInfo.depVarFuncs{newDepVarIdx} = ...
          inline(finalExpr,varName);
    end            
    
    % Prepare for the next iteration
    varIdx = varIdx + 1;
    newDepVarIdx = newDepVarIdx +1;
  end          
end

% And finally, set the freeVarNames part of the varInfo
% struct.
psFuncParLocal.varInfo.freeVarNames =...
    psFuncParLocal.varInfo.optimVarNames;

% Now calculate the depVars (this should of course not really
% change much but it is an interesting check to make.)
psFuncParLocal = psCalcDepVars(psFuncParLocal);
